@@ -3,7 +3,7 @@ import torch
 import pyro
 import pyro.distributions as dist
 from pyro.infer.autoguide import AutoDelta
-from pyro.infer import SVI, TraceMeanField_ELBO, Predictive
+from pyro.infer import SVI, TraceMeanField_ELBO, Trace_ELBO, Predictive
 from pyro.infer.autoguide.initialization import init_to_mean, init_to_value
 from pyro import poutine
 import numpy as np
@@ -271,7 +271,7 @@ class BaseModel:
     @wraps_rp_func(add_isd_results, 
         bar_desc = 'Predicting TF influence', include_factor_data = True)
     def probabilistic_isd(self, model, features,*,hits_matrix, metadata):
-        return model.probabalistic_ISD(features, hits_matrix)
+        return model.probabilistic_isd(features, hits_matrix)
 
 
 class LITE_Model(BaseModel):
